# Copyright 2016 Bela Genge
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

'''
OptimalFlow/mod_emswitch/ModEMSwitch

@author: Bela Genge
'''

from mod_element import ModElement
import mod_network

class ModEMSwitch(ModElement):
    '''
    '''
    INUSE = 'INUSE' # Yes/No-based value in the case that the emulated switch should be used or not
    SWID = 'SWID' # The ID of the switch - can be in HEX
    CTHOST = 'CTHOST' # The controller host
    CTPORT = 'CTPORT' # The controller's port
    REALSW = 'REALSW' # The mapping between the port and the real switch's ID (not DPID)
    REALP = 'REALP' # The mapping between the emulated port and real port

    # Since we have one instance of ModEMSwitch, we can use 'static' attributes.
    ports = {}
    
    def __init__(self, name):
        '''
        Constructor
        '''
        super(ModEMSwitch, self).__init__(name)


    def getEmPorts(self, realmap):
        '''
        Returns the emulated ports associated to the real ports.
        '''
        emmap = []
        for (port, prop) in self.ports.iteritems():
            realSW = self.ports[port][ModEMSwitch.REALSW]
            realPort = self.ports[port][ModEMSwitch.REALP]

            for (swMAC, ports) in realmap.iteritems():
                nodeID = mod_network.of_network.getNodeMAC(swMAC)
                if (nodeID == realSW) and (realPort in ports):
                    emmap.append(port)

        return emmap


of_emsw = None

def initialize():
    global of_emsw
    of_emsw = ModEMSwitch('Controller')
    return of_emsw
