# Copyright 2016 Bela Genge
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

'''
OptimalFlow/mod_flow/ModFlow

@author: Bela Genge
'''

from mod_element import ModElement

class ModFlow(ModElement):
    '''
    '''
    ACC = 'ACC' # Access end-point
    EGG = 'EGG' # Eggress end-point
    DEM = 'DEM' # Demand value
    ACCIP = 'ACCIP' # Access-IP value
    EGGIP = 'EGGIP' # Eggress-IP value
    ACCPORT = 'ACCPORT' # Switch port number where the access end-point is connected
    EGGPORT = 'EGGPORT' # Switch port number where the egress end-point is connected
    PROTO = 'PROTO' # Flow protocol
    P_ARP = 'ARP' # Protol ARP
    P_IP = 'IP' # Protocol IP
    DIR = 'DIR' # Flow direction
    D_BIDIR = 'BIDIR' # Direction bidirectional (requires two flow pushes)
    D_ONEDIR = 'ONEDIR' # Direction onedirectional (requires one flow pushes)
    SEC = 'SEC' # Security property
    ALPHA = 'ALPHA' # Alpha multiplier in the optimization problem - should be very large, but it is also used to implement priority-based selection

    def __init__(self, name):
        '''
        Constructor
        '''
        super(ModFlow, self).__init__(name)

        self.__secur = []
        self.accChanged = False
        self.eggChanged = False


    # Override setProp to make sure that we have certain properties stored
    def setProp(self, prop, name = None):

        # Call function from super-class
        super(ModFlow, self).setProp(prop, name)

        # Now set additional (default) properties
        if not self.hasProp(ModFlow.DIR):
            self.addProp(ModFlow.DIR, ModFlow.D_ONEDIR)

        if not self.hasProp(ModFlow.PROTO):
            self.addProp(ModFlow.PROTO, ModFlow.P_IP)


    def setSecurProp(self, prop):

        self.__secur = prop


    def getSecurProp(self):

        return self.__secur
