# Copyright 2016 Bela Genge
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

'''
OptimalFlow/mod_network/TransientNetwork

@author: Bela Genge
'''

class TransientNetwork(object):
    '''
    '''

    def __init__(self):
        '''
        Constructor
        '''
        self.__paths = {}
        self.__links = {}
        self.__linkcap = {}
        self.__flows = {}
        self.__config1 = {}
        self.__config2 = []
        self.__lid = 0

    
    def addPath(self, id, links, prop):
        
        # Check if the path already exists
        if id in self.__paths:
            return

        # Generate link IDs
        genlinks = []
        for ((n1,n2),cap) in links.iteritems():
            if (n1,n2) not in self.__links:
                if (n2,n1) not in self.__links:
                    self.__links[(n1,n2)] = self.__lid
                    self.__linkcap[self.__lid] = cap
                    genlinks.append(self.__lid)
                    self.__lid += 1
                else:
                    genlinks.append(self.__links[(n2,n1)])
            else:
                genlinks.append(self.__links[(n1,n2)])

        self.__paths[id] = (genlinks,prop)

        print "Added path: ", id, links, prop
        print "Stored paths: ", self.__paths


    def addFlow(self, fid, dem, pr, prop):

        # Add flow configuration
        self.__flows[fid] = (dem, pr, prop)
        print "Added flow: ", fid, (dem, pr, prop)


    def addOrigFlow(self, fid, pid):

        # Add flow in the original configuration to a path
        self.__config1[fid] = pid
        print "Added orig flow: ", fid, pid


    def addOptimFlow(self, fid, pid):

        # Add flow in the optima configuration to a path
        #self.__config2[fid] = pid

        (dem1, pr1, prop1) = self.__flows[fid]
        index = 0
        while index < len(self.__config2):
            (fid2,pid2) = self.__config2[index]
            (dem2, pr2, prop2) = self.__flows[fid2]
            if pr1 > pr2:
                break
            index = index + 1

        self.__config2.insert(index, (fid,pid))

        print "Added optim flow: ", fid, pid


    def _getRemFlows(self, lid):

        # Determines the flows that are going to be moved from this link.
        remf = []
        for (fid,pid) in self.__config1.iteritems():
            (links,prop) = self.__paths[pid]

            if lid in links:
                for (fid1,pid1) in self.__config2:
                    if fid == fid1:
                        (links1,prop1) = self.__paths[pid1]
                        if lid not in links1:
                            remf.append(fid)
                            break

        return remf


    def _getAddFlows(self, lid):

        # Determines the flows that are going to be added to this link.
        addf = []
        for (fid,pid) in self.__config2:
            (links,prop) = self.__paths[pid]

            if lid in links:
                (links1,prop1) = self.__paths[self.__config1[fid]]
                if lid not in links1:
                    addf.append(fid)

        return addf


    def _computeCapDep(self, rescap):

        # Moving of a flow on a link may require the moving of other flows - capacity dependency.
        dep = {}

        # Build the set of flows that are going to be removed for each link
        remlf = {}
        demremlf = {}
        for (lid,lcap) in self.__linkcap.iteritems():
            remlf[lid] = self._getRemFlows(lid)
            demremlf[lid] = 0
            for fid in remlf[lid]:
                (dem,pr,prop) = self.__flows[fid]
                demremlf[lid] += dem

        # Build the set of flows that are going to be moved on each link
        addlf = {}
        demaddlf = {}
        for (lid,lcap) in self.__linkcap.iteritems():
            addlf[lid] = self._getAddFlows(lid)
            demaddlf[lid] = 0
            for fid in addlf[lid]:
                (dem,pr,prop) = self.__flows[fid]
                demaddlf[lid] += dem

        print "*** rem flows: ", remlf
        print "*** add flows: ", addlf

        # Now we are ready to compute the flow dependencies.
        for (fid,pid) in self.__config2:

            dep[fid] = []

            # In the case that this flow does not change path, step to the next one.
            
            pid1 = self.__config1[fid]
            if pid == pid1:
                continue

            (links,prop) = self.__paths[pid] # Optimal path
            (links1,prop1) = self.__paths[pid1] # Original path
            (dem,pr,prop2) = self.__flows[fid] # Flow properties
            for lid in links: # Links in the optimal path

                # In the case that this flow is moved on to this link check that there is enough residual capacity. 
                if lid not in links1:

                    # This is the happiest case, it means that there is sufficient capacity to move all the needed flows on this link.
                    if rescap[lid] > demaddlf[lid]:
                        continue

                    if rescap[lid] - demaddlf[lid] < 0: # This is the case in which we have dependencies with the flows that need to be removed.

                        for rf in remlf[lid]:
                            if rf != fid and rf not in dep[fid]:
                                dep[fid].append(rf)
                        
                        # This is another happy case, since we only need to first remove the necessary flows and then we can add all the needed flows.
                        #if rescap[lid] - addlf[lid] + remlf[lid] > 0:
                            # Append the list of flows
                        #    for rf in remlf[lid]:
                        #        if rf != fid and rf not in dep[fid]:
                        #            dep[fid].append(rf)
                        #else: # This is the worst case when even 

        return dep


    def computeTransient(self):

        tr = []

        # Compute the link residual capacity - the remaining capacity of all links
        rescap = {}
        for (lid,lcap) in self.__linkcap.iteritems():
            rescap[lid] = lcap

            # Loop through each flow in the original configuration
            for (fid,pid) in self.__config1.iteritems():
                (dem,pr,prop) = self.__flows[fid]
                (links,prop1) = self.__paths[pid]

                # Loop through the links of a path
                for lid1 in links:
                    if lid == lid1:
                        rescap[lid] -= dem

        print "*** Residual capacity: ", rescap

        # Build the capacity dependency list. This is the list that says which flows need to be removed first before we can add others.
        capdep = self._computeCapDep(rescap)

        print "*** Dependencies: ", capdep

        # Now build the ordered list of operations based on the capacity dependency and on the flow priorities.
        # Fortunately, the optimal configuration is already ordered based on the priorities.
        for (fid,pid) in self.__config2:

            # This is the best case scenario, meaning that we do not need to move this flow, it is already in place
            pid1 = self.__config1[fid]
            if pid == pid1:
                continue

            # Based on the list of dependencies we simply build the ordered set of operations.
            if len(capdep[fid]) <= 0: # The happiest case, there are no dependencies
                if (fid,'REMOVE') not in tr:
                    tr.append((fid,'REMOVE'))
                tr.append((fid,'ADD'))
                continue

            # When we have dependencies we remove them first.
            for fid1 in capdep[fid]:
                if (fid1,'REMOVE') not in tr:
                    tr.append((fid1,'REMOVE'))

            # Finally, add the delete and add commands for this flow - to minimize disconnection time.
            if (fid, 'REMOVE') not in tr:
                tr.append((fid,'REMOVE'))
            tr.append((fid,'ADD'))            

        return tr
