# Copyright 2011-2013 James McCauley
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at:
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging
import logging.config
from time import sleep

# Important, and needs to be called to initialize instance - inetersting way to create singleton :)
# This needs to be called before all other imports!!!
import core
core = core.initialize()

from pox.datapaths import softwareswitch

class Options (object):
    def set (self, given_name, value):
        name = given_name.replace("-", "_")
        if name.startswith("_") or hasattr(Options, name):
            # Hey, what's that about?
            print("Illegal option:", given_name)
            return False
        has_field = hasattr(self, name)
        has_setter = hasattr(self, "_set_" + name)
        if has_field == False and has_setter == False:
            print("Unknown option:", given_name)
            return False
        if has_setter:
            setter = getattr(self, "_set_" + name)
            setter(given_name, name, value)
        else:
            if isinstance(getattr(self, name), bool):
                # Automatic bool-ization
                value = str_to_bool(value)
            setattr(self, name, value)
        return True

    def process_options (self, options):
        for k,v in options.iteritems():
            if self.set(k, v) is False:
                # Bad option!
                sys.exit(1)


class POXOptions (Options):
    def __init__ (self):
#    self.cli = True
        self.verbose = False
        self.enable_openflow = True
        self.log_config = None

    def _set_h (self, given_name, name, value):
        self._set_help(given_name, name, value)

    def _set_help (self, given_name, name, value):
        print(_help_text)
        #TODO: Summarize options, etc.
        sys.exit(0)

    def _set_version (self, given_name, name, value):
        print(core._get_python_version())
        sys.exit(0)

    def _set_no_openflow (self, given_name, name, value):
        self.enable_openflow = not str_to_bool(value)

#  def _set_no_cli (self, given_name, name, value):
#    self.cli = not str_to_bool(value)

    def _set_log_config (self, given_name, name, value):
        if value is True:
            # I think I use a better method for finding the path elsewhere...
            p = os.path.dirname(os.path.realpath(__file__))
            value = os.path.join(p, "..", "logging.cfg")
        self.log_config = value

    def _set_debug (self, given_name, name, value):
        value = str_to_bool(value)
        if value:
            # Debug implies no openflow and no CLI and verbose
            #TODO: Is this really an option we need/want?
            self.verbose = True
            self.enable_openflow = False

_options = POXOptions()

def _setup_logging ():
    # First do some basic log config...

    # This is kind of a hack, but we need to keep track of the handler we
    # install so that we can, for example, uninstall it later.  This code
    # originally lived in pox.core, so we explicitly reference it here.
    core._default_log_handler = logging.StreamHandler()
    formatter = logging.Formatter(logging.BASIC_FORMAT)
    core._default_log_handler.setFormatter(formatter)
    logging.getLogger().addHandler(core._default_log_handler)
    logging.getLogger().setLevel(logging.INFO)


    # Now set up from config file if specified...
    #TODO:
    #  I think we could move most of the special log stuff into
    #  the log module.  You'd just have to make a point to put the log
    #  module first on the commandline if you wanted later component
    #  initializations to honor it.  Or it could be special-cased?

    if _options.log_config is not None:
        if not os.path.exists(_options.log_config):
            print("Could not find logging config file:", _options.log_config)
            sys.exit(2)
        logging.config.fileConfig(_options.log_config, disable_existing_loggers=True)


def boot_emulated_switch(controller='127.0.0.1', controlPort=6653, swDpid = 'FF', swPorts = 3):
    _setup_logging()
    logging.basicConfig()
    softwareswitch(address=controller, port=controlPort, dpid=swDpid, swPortCount=swPorts)
    core.goUp()
